# Modelowanie przeprowadzone w kolejnosci proponowanej przez J. Farawaya:
# diagnostyka obs odstajacych, transformacje, selekcja cech
# (str 134 "Practical Regression and ANOVA using R" - ksiazka dostepna w sieci)

X=data.frame(state.x77,row.names=state.abb,check.names=T)

# 1. Diagnostyka obserwacji odstajacych - wersja uproszczona
m1=lm(Life.Exp~.,data=X)
x11(); par(mfrow=c(1,2))
res=rstudent(m1); hii=hatvalues(m1)
plot(hii,res,type="n"); abline(h=0,col=2)
text(hii,res,row.names(X),cex=.75)
plot(m1$fit,X[,"Life.Exp"],type="n"); abline(0,1,col=2)
text(m1$fit,X[,"Life.Exp"],row.names(X),cex=.75)
# Testuje teraz hipoteze H0 == "nie ma obserwacji odstajacych" na poziomie 
# alfa=0.1 metoda Bonferroniego, czyli wykonuje ciag testow  T[i] dla hipotez
# H[i] == "i-ta obs nie odstaje" na poziomie alfa/n, gdzie n - liczba obs.
# Wiemy, ze res[i] ~ t(n-p-1), gdzie p - liczba param modelu.
# Zatem do testowania H[i] mozemy wykorzystac (dwustronny) test t-studenta.
# Obliczamy wartosc krytyczna qt(.1/(2*n),n-p-1) = qt(.1/(2*50),50-8-1) = 
# -3.301273. Nastepnie sprawdzamy, ze nie ma residuum przekraczajacego te wartosc.
# Zatem nie odrzucamy H0. Ogolnie: 
# X=X[ -which( abs(res) > -qt(.1/(2*n),n-p-1) ) , ] 

# 2. Transformacje zmiennych
x11(); par(mfrow=c(4,4))
for(i in 1:8)plot(density(X[,i]),main=colnames(X)[i])
for(i in 1:8)plot(density(log(X[,i])),main=paste("log-",colnames(X)[i]))
# widac, ze transformacja log(X) zbliza zmienne 1,3,8 do "normalnosci". Zatem
X[,c(1,3,8)]=log(X[,c(1,3,8)]) 
# Life.Exp zmienie po "konsultacji" z logtrans i boxcox:
colnames(X)[c(1,3,8)]=c("logPopulation","logIlliteracy","logArea")

library(MASS); x11(); par(mfrow=c(1,2))
logtrans(Life.Exp~.,data=X,alpha=seq(-67.5,100,len=1000))
# logtrans uzasadnia, zeby nie przeksztalcac danych
boxcox(Life.Exp~.,data=X,lambda=seq(-2,2,len=100))
# roznica w wiarygodnosci miedzy ident a log jest nieistotna - nie przeksztalcam 

# 3. Selekcja cech
print(round(cor(X),2)) 
# Nie ma cech b ze soba skorelowanych. 
print("**************************")

m2=lm(Life.Exp~.,data=X) 
print(summary(m2))
print("**************************")
# Najistotniejsze sa Murder, HS.Gradale i logPopulation.
# Do takiego samego modelu + najmniej nieistotna Frost prowadzi krokowa 
# optymalizacja bayesowskiego kryterium informacyjnego (BIC).
m3=stepAIC(m2,scope=list(upper=.~.,lower=.~1),trace=T,k=log(nrow(X)))
print("**************************")
print(summary(m3))
# Nastepnie sprawdzam istotnosc pozostalych cech, usuwajac po jednej:
m4=update(m3,.~.-Frost); print(dropterm(m4,test="F")); print(summary(m4))  
print("**************************")
m5=update(m3,.~.-logPopulation);  print(dropterm(m5,test="F")); print(summary(m5))
print("**************************")
m6=update(m5,.~.-Frost); print(dropterm(m6,test="F")); print(summary(m6))  
print("**************************")
# Wybieram model m4, bo ma cor(y,YPred)=.85, gdy wieksze m2 i m3 maja .86
print(round(cor(X[,"Life.Exp"],m4$fit),2)) 



